/*
Lan-Manager V0.01


*/

#include <ucd-snmp/ucd-snmp-config.h>
#include <ucd-snmp/ucd-snmp-includes.h>

//Liste des hte  contacter

struct host {
  char *name;
  char *community;
} hosts[] = {
  { "192.168.1.1",		"public" },
  { "www.laboweb.net",		"public" },
  { NULL }
};



//Variables que l'on va prendre sur ces htes

struct oid {
  char *Name;
  oid Oid[MAX_OID_LEN];
  int OidLen;
} oids[] = {
  { "system.sysDescr.0" },
  { "system.sysUpTime.0" },
  { "interfaces.ifNumber.0" },
  { NULL }
};

//Initialisation

void initialize (void)
{
  struct oid *op = oids;
  
  init_snmp("asynchapp");

  while (op->Name) {
    op->OidLen = sizeof(op->Oid)/sizeof(op->Oid[0]);
    if (!read_objid(op->Name, op->Oid, &op->OidLen)) {
      snmp_perror("read_objid");
      exit(1);
    }
    op++;
  }
}


//On affiche les resultats

int print_result (int status, struct snmp_session *sp, struct snmp_pdu *pdu)
{
  char buf[1024];
  struct variable_list *vp;
  int ix;
  //  struct timeval now;
  // struct timezone tz;
  struct tm *tm;

  //  gettimeofday(&now, &tz);
  //  tm = localtime(&now.tv_sec);
  //  fprintf(stdout, "%.2d:%.2d:%.2d.%.6d ", tm->tm_hour, tm->tm_min, tm->tm_sec,
  //	now.tv_usec);
  switch (status) {
  case STAT_SUCCESS:
    vp = pdu->variables;
    if (pdu->errstat == SNMP_ERR_NOERROR) {
      while (vp) {
        sprint_variable(buf, vp->name, vp->name_length, vp);
        fprintf(stdout, "%s: %s\n", sp->peername, buf);
	vp = vp->next_variable;
      }
    }
    else {
      for (ix = 1; vp && ix != pdu->errindex; vp = vp->next_variable, ix++)
        ;
      if (vp) sprint_objid(buf, vp->name, vp->name_length);
      else strcpy(buf, "(none)");
      fprintf(stdout, "%s: %s: %s\n",
      	sp->peername, buf, snmp_errstring(pdu->errstat));
    }
    return 1;
  case STAT_TIMEOUT:
    fprintf(stdout, "%s: Timeout\n", sp->peername);
    return 0;
  case STAT_ERROR:
    snmp_perror(sp->peername);
    return 0;
  }
  return 0;
}

/*****************************************************************************/


//Initialisation session et cration pdu(message a envoyer)

void synchronous (void)
{
  struct host *hp;

  for (hp = hosts; hp->name; hp++) {
    struct snmp_session ss, *sp;
    struct oid *op;

    snmp_sess_init(&ss);			//Initialisation session
    ss.version = SNMP_VERSION_2c;
    ss.peername = hp->name;
    ss.community = hp->community;
    ss.community_len = strlen(ss.community);
    snmp_synch_setup(&ss);
    if (!(sp = snmp_open(&ss))) {
      snmp_perror("snmp_open");
      continue;
    }
    for (op = oids; op->Name; op++) {
      struct snmp_pdu *req, *resp;
      int status;
      req = snmp_pdu_create(SNMP_MSG_GET);
      snmp_add_null_var(req, op->Oid, op->OidLen);
      status = snmp_synch_response(sp, req, &resp);
      if (!print_result(status, sp, resp)) break;
      snmp_free_pdu(resp);
    }
    snmp_close(sp);
  }
}

/*****************************************************************************/


//structure session

struct session {
  struct snmp_session *sess;		//data
  struct oid *current_oid;	       
} sessions[sizeof(hosts)/sizeof(hosts[0])];

int active_hosts;			//hote(s) restant(s)  traiter


//reception message de l'agent

int asynch_response(int operation, struct snmp_session *sp, int reqid,
		    struct snmp_pdu *pdu, void *magic)
{
  struct session *host = (struct session *)magic;
  struct snmp_pdu *req;

  if (operation == RECEIVED_MESSAGE) {
    if (print_result(STAT_SUCCESS, host->sess, pdu)) {
      host->current_oid++;	      
      if (host->current_oid->Name) {
	req = snmp_pdu_create(SNMP_MSG_GET);
	snmp_add_null_var(req, host->current_oid->Oid, host->current_oid->OidLen);
	if (snmp_send(host->sess, req))
	  return 1;
	else {
	  snmp_perror("snmp_send");
	  snmp_free_pdu(req);
	}
      }
    }
  }
  else
    print_result(STAT_TIMEOUT, host->sess, pdu);

  active_hosts--;
  return 1;
}


/*****************************************************************************/

int main (int argc, char **argv)
{
  initialize();
   
  synchronous();

  return 0;
}
